/*
 * LCD display library for SC1602 module (HITACHI HD44780U controller)
 * 4 bit transfer mode
 * lcd4.c
 */

#include <xc.h>
#include "lcd4.h"

/*
 make additional delay if clock is faster than 28MHz
 E >=140ns
*/
#if _XTAL_FREQ > 28571428
  #define STROBE (LCD4_E=1,asm("nop"),LCD4_E=0)
#else
  #define STROBE (LCD4_E=1,LCD4_E=0)
#endif

/*
 LCD initialization
*/
void Init_LCD4(void){
    LCD4_CONFIGREG = LCD4_CONFIGREG & 0b00001100;	// set TRIS Reg
    /* secure 8bit config */
    __delay_ms(15);	// > 15ms after power on
    LCD4_PORT=0x30;	// E=0, RS=0, set 8bit mode(0x30)
    STROBE;
    __delay_ms(5);	// > 4.1ms
    STROBE;
    __delay_us(100);	// > 100us
    STROBE;
    __delay_us(40);	// > 40us
    LCD4_PORT=0x20; 	// E=0, RS=0, set 4bit mode(0x20)
    STROBE;
    __delay_us(40);	// > 40us
    Cmd_LCD4(0x28);	// 4bit, 2line, 5x7 font
    Cmd_LCD4(DISP_ON);	// display on, prompt=off
    Clr_LCD4;
    Cmd_LCD4(ENT_MODE);	// entry mode, cursor=increment, shift=none
}

/*
 write one byte character or command
 c: ASCII code or command
*/
void W_LCD4(unsigned char c, int rs){
    char low;
    LCD4_RS=rs;                 // set RS
    low = LCD4_PORT & 0x0f;     // save lower 4bit
    LCD4_PORT=(c & 0xf0)+low;	// write high data, keep lower 4bits
    STROBE;
    LCD4_PORT=(c << 4)+low;	// write low data, keep lower 4bits
    STROBE;
    __delay_us(40);		// >37us
}

/*
 write character string
 ptr: start pointer of character string which is NULL terminated.
*/
void Puts_LCD4(unsigned char *ptr){
    while(*ptr){
            Putc_LCD4(*ptr++);
    }
}

/*
 shift display to left n times
*/
void Shift_LN(int n){
    int i;
    for (i=0;i<n;i++){
        Shift_L;
    }
}

/*
 shift display to right n times
*/
void Shift_RN(int n){
    int i;
    for (i=0;i<n;i++){
        Shift_R;
    }
}

/*
 move cursor to left n times
*/
void Cursor_LN(int n){
    int i;
    for (i=0;i<n;i++){
        Cursor_L;
    }
}

/*
 move corsor to right n times
*/
void Cursor_RN(int n){
    int i;
    for (i=0;i<n;i++){
        Cursor_R;
    }
}

/*
 set CG RAM character
 index: character code index from 0 to 8
 ptr: 1st address for 8 bytes character data
*/
void CGRAM_LCD4(unsigned char index, unsigned char *ptr){
    unsigned char i;
    Set_CGA(index << 3);
    for(i=0; i<8; i++){
        Putc_LCD4(*(ptr+i));
    }
}

/*
 set CG RAM character from EEPROM
 index: character code index from 0 to 8
 add: 1st address for 8 bytes character data
*/
void ROM2CGRAM(unsigned char index, unsigned char add){
    unsigned char i;
    Set_CGA(index <<3);
    for(i=0; i<8; i++){
        Putc_LCD4(EEPROM_READ(add+i));
    }
}

