/*
 * LCD display library for SC1602 module (HITACHI HD44780U controller)
 * 4 bit transfer mode
 *
 * File:   lcd4.h
 * Author: Y.Sakazume
 * Created on 2015/05/26, 9:00
 *
 * H/W connection:
 * PIC PORT	LCD Module
 * bit0		LCD E
 * bit1		LCD RS
 * bit2:3	N/C, you can use these port for other purpose
 * bit4:7	LCD Data
 */


#ifndef LCD4_H
#define	LCD4_H

#ifdef	__cplusplus
extern "C" {
#endif

// CONFIG
#define _XTAL_FREQ      10000000      // clock frequency in Hz
#define LCD4_PORT       PORTB         // output port
#define LCD4_CONFIGREG  TRISB         // config register for the output port
#define LCD4_E          PORTBbits.RB0 // output port which is connected E
#define LCD4_RS         PORTBbits.RB1 // output port which is connected RS
// internal macro value
#define CMD             0
#define DATA            1
// LCD command
#define DISP_CLR        0x01        // Clear display
#define MOVE_HOME       0x02        // Return Home
#define ENT_MODE        0x06        // Entry mode set, cursor increment, no shift
#define DISP_ON         0x0C        // Display on, cursor off
#define DISP_OFF        0x08        // Display off
#define SHIFT_L         0x18        // Shift display to left
#define SHIFT_R         0x1C        // Shift display to right
#define CURSOR_L        0x10        // Move cursor to left
#define CURSOR_R        0x14        // Move cursor to right

void W_LCD4(unsigned char, int);                       // write command or character
void Init_LCD4(void);                                  // initialize the LCD
#define Clr_LCD4 (W_LCD4(DISP_CLR, CMD),__delay_ms(2))  // clear display
#define MoveHome_LCD4 (W_LCD4(0x02, CMD),__delay_ms(2)) // move home
#define Putc_LCD4(x) (W_LCD4(x, DATA))                  // write character
#define Cmd_LCD4(x) (W_LCD4(x, CMD))                    // write command
void Puts_LCD4(unsigned char *);                       // write character string
void Shift_LN(int);                                    // shift display to left n times
void Shift_RN(int);                                    // shift display to right n times
#define Shift_L Cmd_LCD4(SHIFT_L);                      // shift display to left
#define Shift_R Cmd_LCD4(SHIFT_R);                      // shift display to right
void Cursor_LN(int);                                   // move cursor to left n times
void Cursor_RN(int);                                   // move corsor to right ntimes
#define Cursor_L Cmd_LCD4(CURSOR_L)                     // move cursor to left
#define Cursor_R Cmd_LCD4(CURSOR_R)                     // move corsor to right
#define Set_DDA(x) Cmd_LCD4(((x)&0x7F)|0x80)            // set DDRAM addr
#define Set_CGA(x) Cmd_LCD4(((x)&0x3F)|0x40)            // set CGRAM addr
void CGRAM_LCD4(unsigned char, unsigned char *);       // set user character to CGRAM
void ROM2CGRAM(unsigned char, unsigned char);       // set user character to CGRAM

#ifdef	__cplusplus
}
#endif

#endif	/* LCD4_H */

